@@ -43,12 +43,12 @@ import itertools
 try:
     # Python 2.x compatibility
     from StringIO import StringIO
-except ImportError:
+except ImportError:  # pragma: PY3
     from io import StringIO
 
 try:
     from types import InstanceType
-except ImportError:
+except ImportError:  # pragma: PY3
     # Python 3.x compatibility
     InstanceType = None
 
@@ -56,26 +56,30 @@ except ImportError:
 __author__ = "Marius Gedminas (marius@gedmin.as)"
 __copyright__ = "Copyright (c) 2008-2017 Marius Gedminas and contributors"
 __license__ = "MIT"
-__version__ = '3.4.0'
-__date__ = '2018-02-13'
+__version__ = '3.5.0'
+__date__ = '2020-10-11'
 
 
 try:
     basestring
-except NameError:
+except NameError:  # pragma: PY3
     # Python 3.x compatibility
     basestring = str
 
 try:
     iteritems = dict.iteritems
-except AttributeError:
+except AttributeError:  # pragma: PY3
     # Python 3.x compatibility
     iteritems = dict.items
 
 IS_INTERACTIVE = False
 try:  # pragma: nocover
     import graphviz
-    if get_ipython().__class__.__name__ != 'TerminalInteractiveShell':
+    if 'TerminalInteractiveShell' not in get_ipython().__class__.__name__:
+        # So far I know two shells where it's inappropriate to use inline
+        # graphics, because they're text only:
+        # - ipython uses a TerminalInteractiveShell
+        # - pycharm's console uses PyDevTerminalInteractiveShell
         IS_INTERACTIVE = True
 except (NameError, ImportError):
     pass
@@ -642,7 +646,8 @@ def find_backref_chain(obj, predicate, max_depth=20, extra_ignore=()):
 
 def show_backrefs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
                   highlight=None, filename=None, extra_info=None,
-                  refcounts=False, shortnames=True, output=None):
+                  refcounts=False, shortnames=True, output=None,
+                  extra_node_attrs=None):
     """Generate an object reference graph ending at ``objs``.
 
     The graph will show you what objects refer to ``objs``, directly and
@@ -675,6 +680,11 @@ def show_backrefs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
     Use ``extra_info`` (a function taking one argument and returning a
     string) to report extra information for objects.
 
+    Use ``extra_node_attrs`` (a function taking the current object as argument,
+    returning a dict of strings) to add extra attributes to the nodes. See
+    https://www.graphviz.org/doc/info/attrs.html for a list of possible node
+    attributes.
+
     Specify ``refcounts=True`` if you want to see reference counts.
     These will mostly match the number of arrows pointing to an object,
     but can be different for various reasons.
@@ -691,6 +701,7 @@ def show_backrefs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
         >>> show_backrefs(obj, filter=lambda x: not inspect.isclass(x))
         >>> show_backrefs(obj, highlight=inspect.isclass)
         >>> show_backrefs(obj, extra_ignore=[id(locals())])
+        >>> show_backrefs(obj, extra_node_attrs=lambda x: dict(URL=str(id(x))))
 
     .. versionchanged:: 1.3
        New parameters: ``filename``, ``extra_info``.
@@ -704,6 +715,8 @@ def show_backrefs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
     .. versionchanged:: 2.0
        New parameter: ``output``.
 
+    .. versionchanged:: 3.5
+       New parameter: ``extra_node_attrs``.
     """
     # For show_backrefs(), it makes sense to stop when reaching a
     # module because you'll end up in sys.modules and explode the
@@ -714,12 +727,14 @@ def show_backrefs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
                        edge_func=gc.get_referrers, swap_source_target=False,
                        filename=filename, output=output, extra_info=extra_info,
                        refcounts=refcounts, shortnames=shortnames,
-                       cull_func=is_proper_module)
+                       cull_func=is_proper_module,
+                       extra_node_attrs=extra_node_attrs)
 
 
 def show_refs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
               highlight=None, filename=None, extra_info=None,
-              refcounts=False, shortnames=True, output=None):
+              refcounts=False, shortnames=True, output=None,
+              extra_node_attrs=None):
     """Generate an object reference graph starting at ``objs``.
 
     The graph will show you what objects are reachable from ``objs``, directly
@@ -752,6 +767,11 @@ def show_refs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
     Use ``extra_info`` (a function returning a string) to report extra
     information for objects.
 
+    Use ``extra_node_attrs`` (a function taking the current object as argument,
+    returning a dict of strings) to add extra attributes to the nodes. See
+    https://www.graphviz.org/doc/info/attrs.html for a list of possible node
+    attributes.
+
     Specify ``refcounts=True`` if you want to see reference counts.
 
     Examples:
@@ -762,6 +782,7 @@ def show_refs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
         >>> show_refs(obj, filter=lambda x: not inspect.isclass(x))
         >>> show_refs(obj, highlight=inspect.isclass)
         >>> show_refs(obj, extra_ignore=[id(locals())])
+        >>> show_refs(obj, extra_node_attrs=lambda x: dict(URL=str(id(x))))
 
     .. versionadded:: 1.1
 
@@ -777,13 +798,16 @@ def show_refs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
 
     .. versionchanged:: 2.0
        New parameter: ``output``.
+
+    .. versionchanged:: 3.5
+       New parameter: ``extra_node_attrs``.
     """
     return _show_graph(objs, max_depth=max_depth, extra_ignore=extra_ignore,
                        filter=filter, too_many=too_many, highlight=highlight,
                        edge_func=gc.get_referents, swap_source_target=True,
                        filename=filename, extra_info=extra_info,
                        refcounts=refcounts, shortnames=shortnames,
-                       output=output)
+                       output=output, extra_node_attrs=extra_node_attrs)
 
 
 def show_chain(*chains, **kw):
@@ -847,8 +871,10 @@ def is_proper_module(obj):
 
     .. versionadded:: 1.8
     """
-    return (inspect.ismodule(obj) and
-            obj is sys.modules.get(getattr(obj, '__name__', None)))
+    return (
+        inspect.ismodule(obj)
+        and obj is sys.modules.get(getattr(obj, '__name__', None))
+    )
 
 
 #
@@ -894,7 +920,7 @@ def _show_graph(objs, edge_func, swap_source_target,
                 max_depth=3, extra_ignore=(), filter=None, too_many=10,
                 highlight=None, filename=None, extra_info=None,
                 refcounts=False, shortnames=True, output=None,
-                cull_func=None):
+                cull_func=None, extra_node_attrs=None):
     if not _isinstance(objs, (list, tuple)):
         objs = [objs]
 
@@ -906,14 +932,14 @@ def _show_graph(objs, edge_func, swap_source_target,
     elif filename and filename.endswith('.dot'):
         f = codecs.open(filename, 'w', encoding='utf-8')
         dot_filename = filename
-    elif IS_INTERACTIVE:
+    elif IS_INTERACTIVE and not filename:
         is_interactive = True
         f = StringIO()
     else:
         fd, dot_filename = tempfile.mkstemp(prefix='objgraph-',
                                             suffix='.dot', text=True)
         f = os.fdopen(fd, "w")
-        if getattr(f, 'encoding', None):
+        if getattr(f, 'encoding', None):  # pragma: PY3
             # Python 3 will wrap the file in the user's preferred encoding
             # Re-wrap it for utf-8
             import io
@@ -946,9 +972,11 @@ def _show_graph(objs, edge_func, swap_source_target,
         # traversing the reference graph backwards.
         target = queue.pop(0)
         tdepth = depth[id(target)]
-        f.write('  %s[label="%s"];\n' % (_obj_node_id(target),
-                                         _obj_label(target, extra_info,
-                                                    refcounts, shortnames)))
+        f.write('  %s[label="%s"%s];\n' % (_obj_node_id(target),
+                                           _obj_label(target, extra_info,
+                                                      refcounts, shortnames),
+                                           _obj_attrs(target,
+                                                      extra_node_attrs)))
         h, s, v = _gradient((0, 0, 1), (0, 0, .3), tdepth, max_depth)
         if inspect.ismodule(target):
             h = .3
@@ -1071,6 +1099,16 @@ def _obj_node_id(obj):
     return ('o%d' % id(obj)).replace('-', '_')
 
 
+def _obj_attrs(obj, extra_node_attrs):
+    if extra_node_attrs is not None:
+        attrs = extra_node_attrs(obj)
+        return ", " + ", ".join('%s="%s"' % (name, _quote(value))
+                                for name, value in sorted(iteritems(attrs))
+                                if value is not None)
+    else:
+        return ""
+
+
 def _obj_label(obj, extra_info=None, refcounts=False, shortnames=True):
     if shortnames:
         label = [_short_typename(obj)]
@@ -1098,7 +1136,7 @@ def _quote(s):
 
 def _get_obj_type(obj):
     objtype = type(obj)
-    if type(obj) == InstanceType:
+    if type(obj) == InstanceType:  # pragma: PY2 -- no old-style classes on PY3
         objtype = obj.__class__
     return objtype
 
@@ -1144,7 +1182,7 @@ def _short_repr(obj):
         name = _name_or_repr(obj.__func__)
         if obj.__self__:
             return name + ' (bound)'
-        else:
+        else:  # pragma: PY2 -- no unbound methods on Python 3
             return name
     # NB: types.LambdaType is an alias for types.FunctionType!
     if _isinstance(obj, types.LambdaType) and obj.__name__ == '<lambda>':
@@ -1171,8 +1209,8 @@ def _gradient(start_color, end_color, depth, max_depth):
 
 
 def _edge_label(source, target, shortnames=True):
-    if (_isinstance(target, dict) and
-            target is getattr(source, '__dict__', None)):
+    if (_isinstance(target, dict)
+            and target is getattr(source, '__dict__', None)):
         return ' [label="__dict__",weight=10]'
     if _isinstance(source, types.FrameType):
         if target is source.f_locals:
