@@ -131,6 +131,16 @@ SINGLE_ELEMENT_OUTPUT = textwrap.dedent('''\
 ''')
 
 
+SINGLE_ELEMENT_OUTPUT_WITH_ATTR = textwrap.dedent('''\
+    digraph ObjectGraph {
+      node[shape=box, style=filled, fillcolor=white];
+      ${label_a}[fontcolor=red];
+      ${label_a}[label="TestObject\\nTestObject(A)", x="y"];
+      ${label_a}[fillcolor="0,0,1"];
+    }
+''')
+
+
 TWO_ELEMENT_OUTPUT = textwrap.dedent('''\
     digraph ObjectGraph {
       node[shape=box, style=filled, fillcolor=white];
@@ -192,6 +202,18 @@ class ShowGraphTest(unittest.TestCase):
                          format(SINGLE_ELEMENT_OUTPUT,
                                 label_a=label))
 
+    def test_with_extra_node_attrs(self):
+        obj = TestObject.get("A")
+        output = StringIO()
+        objgraph._show_graph([obj], edge_function(), False, output=output,
+                             shortnames=True,
+                             extra_node_attrs=lambda o: {'x': 'y'})
+        output_value = output.getvalue()
+        label = objgraph._obj_node_id(obj)
+        self.assertEqual(output_value,
+                         format(SINGLE_ELEMENT_OUTPUT_WITH_ATTR,
+                                label_a=label))
+
     def test_filename_and_output(self):
         output = StringIO()
         self.assertRaises(ValueError, objgraph._show_graph, [],
@@ -411,6 +433,16 @@ class StringRepresentationTest(GarbageCollectedMixin,
             objgraph._obj_label(x, shortnames=False),
             r'mymodule\.MyClass\\n<mymodule\.MyClass object at .*')
 
+    def test_obj_attrs(self):
+        x = object()
+        self.assertEqual(
+            objgraph._obj_attrs(
+                x,
+                lambda o: {'url': 'http://e.com/' + o.__class__.__name__,
+                           'shape': 'diamond',
+                           'ignored': None}),
+            r', shape="diamond", url="http://e.com/object"')
+
     def test_long_typename_with_no_module(self):
         x = type('MyClass', (), {'__module__': None})()
         self.assertEqual('MyClass', objgraph._long_typename(x))
@@ -477,6 +509,11 @@ class StringRepresentationTest(GarbageCollectedMixin,
                          objgraph._gradient((0.1, 0.2, 0.3),
                                             (0.2, 0.3, 0.4), 0, 0))
 
+    def test_edge_label_frame_locals(self):
+        frame = sys._getframe()
+        self.assertEqual(' [label="f_locals",weight=10]',
+                         objgraph._edge_label(frame, frame.f_locals))
+
     @skipIf(sys.version_info[0] > 2, "Python 3 has no unbound methods")
     def test_edge_label_unbound_method(self):
         class MyClass(object):
@@ -486,6 +523,14 @@ class StringRepresentationTest(GarbageCollectedMixin,
                          objgraph._edge_label(MyClass.a_method,
                                               MyClass.a_method.__func__))
 
+    def test_edge_label_bound_method(self):
+        class MyClass(object):
+            def a_method(self):
+                pass
+        self.assertEqual(' [label="__func__",weight=10]',
+                         objgraph._edge_label(MyClass().a_method,
+                                              MyClass().a_method.__func__))
+
     def test_edge_label_long_type_names(self):
         x = type('MyClass', (), {'__module__': 'mymodule'})()
         d = {x: 1}
